CREATE OR REPLACE PACKAGE systemeventlog_pkg IS

  -- Author  : 
  -- Created : 10.04.2016 14:53:10
  -- Purpose :   

  g_host_name VARCHAR2(256);

  g_handle      BOOLEAN := FALSE;
  g_error_stack VARCHAR2(4000);
  g_call_stack  VARCHAR2(4000);

  PROCEDURE p(p_top      IN BOOLEAN DEFAULT TRUE,
              p_msg_type IN VARCHAR2 DEFAULT 'MSG',
              p_module   IN VARCHAR2 DEFAULT NULL,
              p_action   IN VARCHAR2 DEFAULT NULL,
              p_line     IN VARCHAR2 DEFAULT NULL,
              p_msg_xml  IN CLOB DEFAULT empty_clob,
              p_sql_xml  IN CLOB DEFAULT empty_clob,
              p_sys_dt   IN DATE DEFAULT systimestamp);

  PROCEDURE p(p_top      IN BOOLEAN DEFAULT TRUE,
              p_msg_type IN VARCHAR2 DEFAULT 'ERROR',
              p_module   IN VARCHAR2 DEFAULT $$PLSQL_UNIT,
              p_action   IN VARCHAR2 DEFAULT NULL,
              p_line     IN VARCHAR2 DEFAULT NULL,
              p_param1   IN VARCHAR2 DEFAULT NULL,
              p_value1   IN VARCHAR2 DEFAULT NULL,
              p_param2   IN VARCHAR2 DEFAULT NULL,
              p_value2   IN VARCHAR2 DEFAULT NULL,
              p_param3   IN VARCHAR2 DEFAULT NULL,
              p_value3   IN VARCHAR2 DEFAULT NULL,
              p_param4   IN VARCHAR2 DEFAULT NULL,
              p_value4   IN VARCHAR2 DEFAULT NULL,
              p_param5   IN VARCHAR2 DEFAULT NULL,
              p_value5   IN VARCHAR2 DEFAULT NULL,
              p_param6   IN VARCHAR2 DEFAULT NULL,
              p_value6   IN VARCHAR2 DEFAULT NULL,
              p_param7   IN VARCHAR2 DEFAULT NULL,
              p_value7   IN VARCHAR2 DEFAULT NULL,
              p_param8   IN VARCHAR2 DEFAULT NULL,
              p_value8   IN VARCHAR2 DEFAULT NULL,
              p_param9   IN VARCHAR2 DEFAULT NULL,
              p_value9   IN VARCHAR2 DEFAULT NULL,
              p_param10  IN VARCHAR2 DEFAULT NULL,
              p_value10  IN VARCHAR2 DEFAULT NULL,
              p_sql_xml  IN CLOB DEFAULT empty_clob,
              p_dt       IN DATE DEFAULT systimestamp);

END systemeventlog_pkg;
/
CREATE OR REPLACE PACKAGE BODY systemeventlog_pkg IS

  FUNCTION f_sysevent RETURN VARCHAR2 IS
  BEGIN
    RETURN sys.dbms_standard.sysevent;
  END;

  FUNCTION f_username RETURN VARCHAR2 IS
    v_username sys.user_users.username%TYPE;
  BEGIN
    SELECT s.osuser
      INTO v_username
      FROM sys.v_$session s
      JOIN sys.v_$mystat m
        ON m.sid = s.sid
     WHERE rownum = 1;
  
    RETURN v_username;
  EXCEPTION
    WHEN OTHERS THEN
      RETURN 'UNK';
  END;

  PROCEDURE p_handleall(p_top IN BOOLEAN) IS
  BEGIN
    IF p_top THEN
      g_handle      := FALSE;
      g_error_stack := NULL;
      g_call_stack  := NULL;
    ELSIF NOT g_handle THEN
      g_handle      := TRUE;
      g_error_stack := sys.dbms_utility.format_error_stack;
      g_call_stack  := sys.dbms_utility.format_call_stack;
    END IF;
  END p_handleall;

  PROCEDURE p(p_top      IN BOOLEAN DEFAULT TRUE,
              p_msg_type IN VARCHAR2 DEFAULT 'MSG',
              p_module   IN VARCHAR2 DEFAULT NULL,
              p_action   IN VARCHAR2 DEFAULT NULL,
              p_line     IN VARCHAR2 DEFAULT NULL,
              p_msg_xml  IN CLOB DEFAULT empty_clob,
              p_sql_xml  IN CLOB DEFAULT empty_clob,
              p_sys_dt   IN DATE DEFAULT systimestamp) IS
    v_username sys.user_users.username%TYPE;
    PRAGMA AUTONOMOUS_TRANSACTION;
  BEGIN
    IF p_top = TRUE THEN
      p_handleall(TRUE);
      v_username := f_username;
    ELSE
      p_handleall(FALSE);
    END IF;
  
    INSERT INTO miravild.systemeventlog l
      (l.sys_dt,
       l.module,
       l.action,
       l.line,
       l.msg_type,
       l.msg_xml,
       l.sessionid,
       l.login_user,
       l.database_name,
       l.username,
       l.sql_fulltext,
       l.error_stack,
       l.call_stack,
       l.authentication_type,
       l.current_schema,
       l.db_name,
       l.os_user,
       l.host,
       l.ip_address)
    VALUES
      (nvl(p_sys_dt, systimestamp),
       substr(nvl(upper(p_module), 'ANONYMOUS'), 1, 256),
       substr(upper(p_action), 1, 256),
       p_line,
       p_msg_type,
       p_msg_xml,
       sys_context('USERENV', 'SESSIONID'),
       sys.login_user,
       g_host_name,
       v_username,
       p_sql_xml,
       g_error_stack,
       g_call_stack,
       sys_context('USERENV', 'AUTHENTICATION_TYPE'),
       sys_context('USERENV', 'CURRENT_SCHEMA'),
       sys_context('USERENV', 'DB_NAME'),
       sys_context('USERENV', 'OS_USER'),
       sys_context('USERENV', 'HOST'),
       sys_context('USERENV', 'IP_ADDRESS'));
    COMMIT WORK;
  
  EXCEPTION
    WHEN OTHERS THEN
      RAISE;
  END p;

  PROCEDURE p(p_top      IN BOOLEAN DEFAULT TRUE,
              p_msg_type IN VARCHAR2 DEFAULT 'ERROR',
              p_module   IN VARCHAR2 DEFAULT $$PLSQL_UNIT,
              p_action   IN VARCHAR2 DEFAULT NULL,
              p_line     IN VARCHAR2 DEFAULT NULL,
              p_param1   IN VARCHAR2 DEFAULT NULL,
              p_value1   IN VARCHAR2 DEFAULT NULL,
              p_param2   IN VARCHAR2 DEFAULT NULL,
              p_value2   IN VARCHAR2 DEFAULT NULL,
              p_param3   IN VARCHAR2 DEFAULT NULL,
              p_value3   IN VARCHAR2 DEFAULT NULL,
              p_param4   IN VARCHAR2 DEFAULT NULL,
              p_value4   IN VARCHAR2 DEFAULT NULL,
              p_param5   IN VARCHAR2 DEFAULT NULL,
              p_value5   IN VARCHAR2 DEFAULT NULL,
              p_param6   IN VARCHAR2 DEFAULT NULL,
              p_value6   IN VARCHAR2 DEFAULT NULL,
              p_param7   IN VARCHAR2 DEFAULT NULL,
              p_value7   IN VARCHAR2 DEFAULT NULL,
              p_param8   IN VARCHAR2 DEFAULT NULL,
              p_value8   IN VARCHAR2 DEFAULT NULL,
              p_param9   IN VARCHAR2 DEFAULT NULL,
              p_value9   IN VARCHAR2 DEFAULT NULL,
              p_param10  IN VARCHAR2 DEFAULT NULL,
              p_value10  IN VARCHAR2 DEFAULT NULL,
              p_sql_xml  IN CLOB DEFAULT empty_clob,
              p_dt       IN DATE DEFAULT systimestamp) IS
  BEGIN
    miravild.systemeventlog_pkg.p(p_top      => p_top,
                                  p_sys_dt   => p_dt,
                                  p_msg_type => p_msg_type,
                                  p_module   => p_module,
                                  p_action   => p_action,
                                  p_line     => p_line,
                                  p_msg_xml  => to_clob('<msg>' || chr(10) ||
                                                        (CASE
                                                          WHEN p_param1 IS NOT NULL THEN
                                                           '<param "' || p_param1 || '">' ||
                                                           p_value1 || '</param>' ||
                                                           chr(10)
                                                        END) || (CASE
                                                          WHEN p_param2 IS NOT NULL THEN
                                                           '<param "' || p_param2 || '">' ||
                                                           p_value2 || '</param>' ||
                                                           chr(10)
                                                        END) || (CASE
                                                          WHEN p_param3 IS NOT NULL THEN
                                                           '<param "' || p_param3 || '">' ||
                                                           p_value3 || '</param>' ||
                                                           chr(10)
                                                        END) || (CASE
                                                          WHEN p_param4 IS NOT NULL THEN
                                                           '<param "' || p_param4 || '">' ||
                                                           p_value4 || '</param>' ||
                                                           chr(10)
                                                        END) || (CASE
                                                          WHEN p_param5 IS NOT NULL THEN
                                                           '<param "' || p_param5 || '">' ||
                                                           p_value5 || '</param>' ||
                                                           chr(10)
                                                        END) || (CASE
                                                          WHEN p_param6 IS NOT NULL THEN
                                                           '<param "' || p_param6 || '">' ||
                                                           p_value6 || '</param>' ||
                                                           chr(10)
                                                        END) || (CASE
                                                          WHEN p_param7 IS NOT NULL THEN
                                                           '<param "' || p_param7 || '">' ||
                                                           p_value7 || '</param>' ||
                                                           chr(10)
                                                        END) || (CASE
                                                          WHEN p_param8 IS NOT NULL THEN
                                                           '<param "' || p_param8 || '">' ||
                                                           p_value8 || '</param>' ||
                                                           chr(10)
                                                        END) || (CASE
                                                          WHEN p_param9 IS NOT NULL THEN
                                                           '<param "' || p_param9 || '">' ||
                                                           p_value9 || '</param>' ||
                                                           chr(10)
                                                        END) || (CASE
                                                          WHEN p_param10 IS NOT NULL THEN
                                                           '<param "' || p_param10 || '">' ||
                                                           p_value10 || '</param>' ||
                                                           chr(10)
                                                        END) || '</msg>'),
                                  p_sql_xml  => to_clob(p_sql_xml));
  EXCEPTION
    WHEN OTHERS THEN
      RAISE;
  END p;

BEGIN
  SELECT i.host_name INTO g_host_name FROM sys.v_$instance i;
EXCEPTION
  WHEN OTHERS THEN
    RAISE;
  
END systemeventlog_pkg;
/
